"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.render = render;

var _defineProperty2 = _interopRequireDefault(require("@babel/runtime/helpers/defineProperty"));

var _objectWithoutProperties2 = _interopRequireDefault(require("@babel/runtime/helpers/objectWithoutProperties"));

var _Component = require("../Component");

var _Node = require("../Node");

function ownKeys(object, enumerableOnly) { var keys = Object.keys(object); if (Object.getOwnPropertySymbols) { var symbols = Object.getOwnPropertySymbols(object); if (enumerableOnly) symbols = symbols.filter(function (sym) { return Object.getOwnPropertyDescriptor(object, sym).enumerable; }); keys.push.apply(keys, symbols); } return keys; }

function _objectSpread(target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i] != null ? arguments[i] : {}; if (i % 2) { ownKeys(Object(source), true).forEach(function (key) { (0, _defineProperty2.default)(target, key, source[key]); }); } else if (Object.getOwnPropertyDescriptors) { Object.defineProperties(target, Object.getOwnPropertyDescriptors(source)); } else { ownKeys(Object(source)).forEach(function (key) { Object.defineProperty(target, key, Object.getOwnPropertyDescriptor(source, key)); }); } } return target; }

var ATTRIBUTES_MAP = {
  className: 'class',
  htmlFor: 'for'
};

function isValue(data) {
  return typeof data === 'string' || typeof data === 'number';
}

function isVoid(data) {
  return data === null || data === undefined || typeof data === 'boolean';
}

function isNode(data) {
  return data instanceof _Node.Node;
}

function isComponent(data) {
  return data.prototype instanceof _Component.Component;
}

function isNameEvent(name) {
  return /^on[A-Z][A-Za-z]+$/.test(name);
}

function toHTMLElementEventName(name) {
  return name.replace(/^on/, '').toLowerCase();
}

function setupHTMLElementAttributes(htmlElement, attributes) {
  Object.keys(attributes).forEach(function (key) {
    var name = ATTRIBUTES_MAP[key] || key;
    var value = attributes[key];

    if (name === 'style') {
      if (!value) return;
      Object.keys(value).forEach(function (styleKey) {
        var styleValue = value[styleKey];
        htmlElement.style[styleKey] = styleValue;
      });
    } else if (name === 'dangerouslySetInnerHTML') {
      if (value && value.__html) {
        htmlElement.innerHTML = value.__html;
      }
    } else if (isNameEvent(name)) {
      var eventName = toHTMLElementEventName(name);
      htmlElement.addEventListener(eventName, value);
    } else {
      htmlElement.setAttribute(name, value);
    }
  });
}

function setupHTMLElementChildren(htmlElement, children) {
  children.forEach(function (child) {
    var childChunk;

    if (isNode(child)) {
      childChunk = walk(child);
    } else if (isValue(child)) {
      childChunk = document.createTextNode(child);
    } else if (isVoid(child)) {
      return;
    } else if (Array.isArray(child)) {
      setupHTMLElementChildren(htmlElement, child);
      return;
    } else {
      throw new Error('Invalid children.');
    }

    htmlElement.appendChild(childChunk);
  });
}

function setupHTMLElement(element, attributes, children) {
  var htmlElement = document.createElement(element);
  setupHTMLElementAttributes(htmlElement, attributes);
  setupHTMLElementChildren(htmlElement, children);
  return htmlElement;
}

function walk(node) {
  if (!isNode(node)) {
    throw new Error('Invalid node.');
  }

  var _node$attributes = node.attributes,
      ref = _node$attributes.ref,
      attributes = (0, _objectWithoutProperties2.default)(_node$attributes, ["ref"]);
  var isHTMLElement = typeof node.element === 'string';
  var isComposedElement = typeof node.element === 'function';
  var chunk;

  if (isHTMLElement) {
    chunk = setupHTMLElement(node.element, attributes, node.children);
    ref && ref(chunk);
  } else if (isComposedElement) {
    // If children is only one element, we pass it directly to simplify data manipulation.
    var children = node.children.length === 1 ? node.children[0] : node.children;

    var props = _objectSpread(_objectSpread({}, node.attributes), {}, {
      children: children
    });

    var nextNode;
    var toReference;

    if (isComponent(node.element)) {
      var instance = new node.element(props); // eslint-disable-line new-cap

      nextNode = instance.render();
      toReference = instance;
    } else {
      nextNode = node.element(props);
    }

    chunk = walk(nextNode);
    ref && ref(toReference || chunk);
  } else {
    throw new Error('Invalid element.');
  }

  return chunk;
}

function render(parentNode, parentRoot) {
  var dom = walk(parentNode);
  parentRoot.appendChild(dom);
}